@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import division, absolute_import, print_function, unicode_literals
 from datetime import datetime
-import re
 import requests
 import json
 import logging
@@ -49,6 +48,11 @@ class API(object):
         request.update({'type': action, 'handler': url})
 
         return self.client.request(request)
+    
+    def _http_url(self, action, url, request={}):
+        request.update({"type": action, "handler": url})
+
+        return self.client.request_url(request)
 
     def _http_stream(self, action, url, dest_file, request={}):
         request.update({'type': action, 'handler': url})
@@ -58,6 +62,9 @@ class API(object):
     def _get(self, handler, params=None):
         return self._http("GET", handler, {'params': params})
 
+    def _get_url(self, handler, params=None):
+        return self._http_url("GET", handler, {"params": params})
+
     def _post(self, handler, json=None, params=None):
         return self._http("POST", handler, {'params': params, 'json': json})
 
@@ -76,6 +83,25 @@ class API(object):
     def try_server(self):
         return self._get("System/Info/Public")
 
+    def command(self, id, command, params=None, json=None):
+        return self._post(
+            "Sessions/%s/Command" % id,
+            json={"Name": command, "Arguments": json},
+            params=params,
+        )
+
+    def remote(self, id, command, params=None, json=None):
+        handler = (
+            "Sessions/%s/Playing/%s" % (id, command)
+            if command
+            else "Sessions/%s/Playing" % id
+        )
+        return self._post(
+            handler,
+            json=json,
+            params=params,
+        )
+
     def sessions(self, handler="", action="GET", params=None, json=None):
         if action == "POST":
             return self._post("Sessions%s" % handler, json, params)
@@ -110,10 +136,41 @@ class API(object):
         return self._get("Videos%s" % handler)
 
     def artwork(self, item_id, art, max_width, ext="jpg", index=None):
-        if index is None:
-            return jellyfin_url(self.client, "Items/%s/Images/%s?MaxWidth=%s&format=%s" % (item_id, art, max_width, ext))
+        params = {"MaxWidth": max_width, "format": ext}
+        handler = ("Items/%s/Images/%s" % (item_id, art) if index is None
+            else "Items/%s/Images/%s/%s" % (item_id, art, index)
+        )
 
-        return jellyfin_url(self.client, "Items/%s/Images/%s/%s?MaxWidth=%s&format=%s" % (item_id, art, index, max_width, ext))
+        return self._get_url(handler, params)
+
+    def audio_url(self, item_id, container=None, audio_codec=None, max_streaming_bitrate=140000000):
+        params = {
+            "UserId": "{UserId}",
+            "DeviceId": "{DeviceId}",
+            "MaxStreamingBitrate": max_streaming_bitrate,
+        }
+
+        if container:
+            params["Container"] = container
+
+        if audio_codec:
+            params["AudioCodec"] = audio_codec
+
+        return self._get_url("Audio/%s/universal" % item_id, params)
+
+    def video_url(self, item_id, media_source_id=None):
+        params = {
+            "static": "true",
+            "DeviceId": "{DeviceId}"
+        }
+        if media_source_id is not None:
+            params["MediaSourceId"] = media_source_id
+
+        return self._get_url("Videos/%s/stream" % item_id, params)
+
+    def download_url(self, item_id):
+        params = {}
+        return self._get_url("Items/%s/Download" % item_id, params)
 
     #################################################################################################
 
@@ -344,6 +401,56 @@ class API(object):
     def session_stop(self, data):
         return self.sessions("/Playing/Stopped", "POST", json=data)
 
+    def remote_pause(self, id):
+        return self.remote(id, "Pause")
+
+    def remote_playpause(self, id):
+        return self.remote(id, "PlayPause")
+
+    def remote_seek(self, id, ticks, params={}, json={}):
+        """Set the volume on the sessions.
+        
+            @id: The session id to control
+            @ticks: The position (in ticks) to seek to"""
+        return self.remote(
+            id, "Seek", params={"seekPositionTicks": ticks, **params}, json=json
+        )
+
+    def remote_stop(self, id):
+        return self.remote(id, "Stop")
+
+    def remote_unpause(self, id):
+        return self.remote(id, "Unpause")
+
+    def remote_play_media(
+        self, id: str, item_ids: list[str], command: str = "PlayNow", params={}, json={}
+    ):
+        """Instruct the session to play some media
+        
+            @id: The session id to control
+            @item_ids: A list of items to play
+            @command: When to play. (*PlayNow*, PlayNext, PlayLast, PlayInstantMix, PlayShuffle)
+        """
+        return self.remote(
+            id,
+            None,
+            json=json,
+            params={"playCommand": command, "itemIds": item_ids, **params},
+        )
+
+    def remote_set_volume(self, id: str, volume: int, json={}):
+        """Set the volume on the sessions.
+        
+            @id: The session id to control
+            @volume: The volume normalized from 0 to 100"""
+        return self.command(id, "SetVolume", json={"Volume": volume, **json})
+
+    def remote_mute(self, id):
+        return self.command(id, "Mute")
+
+    def remote_unmute(self, id):
+        return self.command(id, "Unmute")
+
     def item_played(self, item_id, watched):
         return self.users("/PlayedItems/%s" % item_id, "POST" if watched else "DELETE")
 
@@ -396,7 +503,8 @@ class API(object):
             'DeviceId': "{DeviceId}",
             'PlaySessionId': play_id,
             'Container': container,
-            'AudioCodec': audio_codec
+            'AudioCodec': audio_codec,
+            "MaxStreamingBitrate": max_streaming_bitrate,
         })
 
     def get_default_headers(self):
@@ -435,7 +543,6 @@ class API(object):
 
         return request_method(url, **request_settings)
 
-
     def login(self, server_url, username, password=""):
         path = "Users/AuthenticateByName"
         authData = {
@@ -465,8 +572,6 @@ class API(object):
         return {}
 
     def validate_authentication_token(self, server):
-
-        url = "%s/%s" % (server['address'], "system/info")
         authTokenHeader = {
                     'X-MediaBrowser-Token': server['AccessToken']
                 }
